
package org.mule.modules.github.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.github.processors.GetIssuesMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class GetIssuesDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public GetIssuesDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(GetIssuesMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("user")!= null)&&(!StringUtils.isBlank(element.getAttribute("user")))) {
            builder.addPropertyValue("user", element.getAttribute("user"));
        }
        if ((element.getAttribute("repository")!= null)&&(!StringUtils.isBlank(element.getAttribute("repository")))) {
            builder.addPropertyValue("repository", element.getAttribute("repository"));
        }
        Element filterDataListElement = null;
        filterDataListElement = DomUtils.getChildElementByTagName(element, "filter-data");
        List<Element> filterDataMapChilds = null;
        if (filterDataListElement!= null) {
            String filterDataRef = filterDataListElement.getAttribute("ref");
            if ((filterDataRef!= null)&&(!StringUtils.isBlank(filterDataRef))) {
                if ((!filterDataRef.startsWith(patternInfo.getPrefix()))&&(!filterDataRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("filterData", new RuntimeBeanReference(filterDataRef));
                } else {
                    builder.addPropertyValue("filterData", filterDataRef);
                }
            } else {
                ManagedMap filterData = new ManagedMap();
                filterDataMapChilds = DomUtils.getChildElementsByTagName(filterDataListElement, "filter-datum");
                if (filterDataMapChilds!= null) {
                    if (filterDataMapChilds.size() == 0) {
                        filterDataMapChilds = DomUtils.getChildElements(filterDataListElement);
                    }
                    for (Element filterDataChild: filterDataMapChilds) {
                        String filterDataValueRef = filterDataChild.getAttribute("value-ref");
                        String filterDataKeyRef = filterDataChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((filterDataValueRef!= null)&&(!StringUtils.isBlank(filterDataValueRef))) {
                            if ((!filterDataValueRef.startsWith(patternInfo.getPrefix()))&&(!filterDataValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(filterDataValueRef);
                            } else {
                                valueObject = filterDataValueRef;
                            }
                        } else {
                            valueObject = filterDataChild.getTextContent();
                        }
                        if ((filterDataKeyRef!= null)&&(!StringUtils.isBlank(filterDataKeyRef))) {
                            keyObject = new RuntimeBeanReference(filterDataKeyRef);
                        } else {
                            keyObject = filterDataChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = filterDataChild.getTagName();
                        }
                        filterData.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("filterData", filterData);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
